/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.Fg003Typ076
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg003Typ076 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg003Typ076 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.Fg003Typ076.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.Fg003Typ076.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg003Typ076 ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg003Typ076 wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7.conversion;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.tls.osi7.De;
import de.bsvrz.kex.tls.osi7.DeBlock;
import de.bsvrz.kex.tls.osi7.Eak;
import de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Umsetzungsklasse für FG 3, Typ 76 Erweiterung: unterstützt auch nachgelieferte Telegramme (Standard ID+32)
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 */

public class Fg003Typ076 extends Fg003Default {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Sonstige ---------------------------------------------------------------
	/** DebugLogger für Debug-Ausgaben */
	private static final Debug debug = Debug.getLogger();
// Private ----------------------------------------------------------------
	/**
	 * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat folgenden
	 * Aufbau:
	 * <pre>
	 * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
	 * z.B.
	 * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "1", "29", "2", "typ.deLve"
	 * </pre>
	 * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
	 * Empfangsanmeldung als Empfänger.</li> <li> In der Methode {@link #update} wird für jede zur
	 * Attributgruppen/Aspket-Kombination passende Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag
	 * wie folgt gebildet wird (hier für den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br>
	 * Dabei enthält <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li> <li> Für jede
	 * Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfänger. Sind mehrere passende Eintrage
	 * vorhanden, werden entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch können mehrere ATG/ASP-Kombinationen in
	 * einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eines Fachmodells):</li>
	 * <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
	 */
	private final Object[][] davToTls = new Object[][]{
			// Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
	};

	/**
	 * Tabelle, in der das Verhalten beim Empfang von TLS De-Blöcken eingetragen wird. Jede Tabellenzeile hat folgenden
	 * Aufbau:
	 * <pre>
	 * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
	 * z.B.
	 * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "1", "29", "2", "typ.deLve"
	 * </pre>
	 * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
	 * Sendeanmeldung als Quelle.</li> <li> In der Methode {@link #processDeBlock} wird für jede zum De-Block passende
	 * Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier für
	 * den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock, Data,
	 * DataDescription)</code><br> Dabei enthält der <code>deBlock</code> die Daten des zu verarbeitenden De-Blocks und
	 * <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem direkt die Inhalte des
	 * De-Blocks zugewiesen werden können.<code>DataDescription</code> enthält die passende {@link
	 * DataDescription} für die zu sendenden Daten.</li> <li> Für jede Attributguppen/Aspekt
	 * Kombination erfolgt eine Sendeanmeldung als Quelle. Sind mehrere passende Eintrage vorhanden, werden entsprechend
	 * mehrere Methodenaufrufe durchgeführt. Dadurch kann ein De-Block mehrfach ausgewert und an verschiedene
	 * Attributgruppen weitergerleitet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eine
	 * Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
	 */

	private final Object[][] tlsToDav = new Object[][]{
			{"atg.tlsUfdErgebnisMeldungEisFilmDickeEFD", "asp.tlsAntwort", "3", "76", "4", "typ.deUfd"},
			{"atg.tlsUfdErgebnisMeldungEisFilmDickeEFD", "asp.tlsAntwort", "3", "76", "36", "typ.deUfd"},
	        {"atg.ufdsEisFilmDicke", "asp.externeErfassung", "3", "76", "4", "typ.ufdsEisFilmDicke"}
	};

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Objekterzeugung mit Anmeldung der globalen und lokalen Umsetzungseinträge
	 *
	 * @param daf DaV
	 * @param eak Eak
	 * @param de De
	 */
	public Fg003Typ076(ClientDavInterface daf, Eak eak, De de) {
		super(daf, eak, de);
		subscribeAllData(tlsToDav, davToTls);
	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Entspricht {@link Fg003Typ076#Fg003Typ076Id004AtgTlsUfdErgebnisMeldungEisFilmDickeEFDAspTlsAntwort(DeBlock,
	 *Data,DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ076Id036AtgTlsUfdErgebnisMeldungEisFilmDickeEFDAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// Generierte Methode: Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
		ResultData resultData = Fg003Typ076Id004AtgTlsUfdErgebnisMeldungEisFilmDickeEFDAspTlsAntwort(deBlock, data, dataDescription);
		return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
	}

	  /**
	   * Umsetzung von TLS-DE-Block in logischen Umfelddatensensor.
	   *
	   * @param deBlock Der zu bearbeitende De-Block.
	   * @param data Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	   * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	   * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten
	   *     Daten.
	   */
	  public ResultData Fg003Typ076Id004AtgUfdsEisFilmDickeAspExterneErfassung(
	      DeBlock deBlock, Data data, DataDescription dataDescription) {

	    // DE-Daten-Ausfallüberwachung aufrufen und nächsten Zeitpunkt eintragen, an dem spätesten KEINE
	    // DATEN gesendet wird.
	    DeAusfallUeberwachung.addAuftrag(
	        _config,
	        _de,
	        dataDescription,
	        "typ.ufdsEisFilmDicke",
	        _eakConversionModul.getTimeStamp(),
	        _intervallDauerUFD);

	    // _Messwert_ aus Telegramm
	    // -------------------------------------------------------------------------------
	    int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

	    return tlsDatenNachResultUfdLogisch(
	        _Messwert_, data, dataDescription, "EisFilmDicke", 255, _intervallDauerUFD);
	  }
}
